//---------------------------------------------------------------------------

#ifndef ApplicationIoH
#define ApplicationIoH

#include <IniFile_Mb.h>
#include <SoftwareTimer_Mb.h>
#include <StopWatch_Mb.h>
#include "Target.h"
#include <fstream>

//==============================================================================
//  CLASS IUserInterface - Interface to communicate back to UI
//==============================================================================

class IUserInterface
{
public:
    virtual ~IUserInterface()  {  }
    virtual void  Log(const std::string & msg) = 0;
    virtual void  OnProgress(int percent) = 0;
    virtual void  OnLoadMessage(const std::string & msg) = 0;
    virtual void  OnLoadComplete(unsigned int result) = 0;
    virtual void  OnElapsed(const std::string & msg) = 0;
    virtual void  OnRevision(const std::string & msg) = 0;
};

//==============================================================================
//  CLASS ApplicationSettings - Remember Settings between runs
//==============================================================================

class ApplicationSettings
{
public:
    // Ctor
    ApplicationSettings();
	~ApplicationSettings();

    //  Data
    int         Target;
    bool        Debug;

    int         DebugLevel;
    bool        LogFile;
    bool        LogMessages;
    std::string XsvfFile;

    std::string Path()
        {  return FPath;  }

private:
    // Data
    Innovative::IniFile     * Ini;
    std::string             FPath;
};


//==============================================================================
//  CLASS ApplicationIo  -- Hardware Access and Application Io Class
//==============================================================================

class ApplicationIo
{
public:
	//
    //  Member Functions
    ApplicationIo(IUserInterface * ui);
    ~ApplicationIo();

    void   Open(int target);
    void   Close();

    void   Load();

    unsigned int  BoardCount();
    std::string   BoardName();

    Innovative::PciLogicVersionInfoIntf &  LogicVersion();

    ii32   PeekPort(int offset);
    void   PokePort(int offset, ii32 value);

    ApplicationSettings         Settings;

private:
    //
    //  Member Data
    IUserInterface *            UI;
    PmcModule                   Baseboard;
    Innovative::SoftwareTimer   Timer;
    Innovative::StopWatch       Watch;
    std::ofstream               Log;

    // Event Handlers
    void            HandleLoadProgress(Innovative::ProcessProgressEvent & event);
    void            HandleLoadComplete(Innovative::ProcessCompletionEvent & event);
    void            HandleLoadMessage(Innovative::ProcessStatusEvent & event);
    void            HandleTimer(OpenWire::NotifyEvent & event);
    // Support
    void  DisplayLogicVersion();
};


#endif
