//---------------------------------------------------------------------------

#ifndef ApplicationIoH
#define ApplicationIoH

#include <IniFile_Mb.h>
#include <SoftwareTimer_Mb.h>
#include "Target.h"

//==============================================================================
//  CLASS IUserInterface - Interface to communicate back to UI
//==============================================================================

class IUserInterface
{
public:
    virtual ~IUserInterface()  {  }
    virtual void  Log(const std::string & a_string) = 0;
};

//==============================================================================
//  CLASS ApplicationSettings - Remember Settings between runs
//==============================================================================

class ApplicationSettings
{
public:
    // Ctor
    ApplicationSettings();
	~ApplicationSettings();

	void 		Read();
	void 		Write();

    //  Data
    int     Target;

private:
    // Data
    Innovative::IniFile     * Ini;
};


//==============================================================================
//  CLASS ApplicationIo  -- Hardware Access and Application Io Class
//==============================================================================

class ApplicationIo
{
public:
	//
    //  Member Functions
    ApplicationIo(IUserInterface * ui);
    ~ApplicationIo();

    void   Open(int target);
    bool   IsOpen() const
        {  return FOpen;  }
    void   Close();

    unsigned int  BoardCount();
    void LicenseCheck();

    std::string   BoardName();

    void   Led(bool state);
    void   ToggleLed();

    // Properties
    void   Blink(bool state);
    bool   Blink() const;

    ApplicationSettings     Settings;

private:
    //
    //  Member Data
    IUserInterface *            UI;
    PmcModule                   Baseboard;
    Innovative::SoftwareTimer   Timer;
    bool                        FOpen;

    void    HandleTimer(OpenWire::NotifyEvent & event);
};


#endif
