#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <Framework/Qt/GuiXfr_Qt.h>
#include <Malibu_Mb.h>

using namespace Innovative;
using namespace std;

// Set to zero while adding event handlers to work-around
// QtCreator bug which adds slots here instead of form header
#if 1
//===========================================================================
//  CLASS GuiInterface  -- Interface for callbacks to UI from App Logic
//===========================================================================

class UserInterface : public IUserInterface
{
public:
    // Ctor
    UserInterface(MainWindow* form)
        : Form(form)
        {  }

    // Methods
    virtual void  Log(const std::string & a_string)
        {  Form->Log(a_string);  }
    virtual void  OnProgress(int percent)
        {  Form->OnProgress(percent);  }
    virtual void  OnParseComplete()
        {  Form->OnParseComplete();  }
    virtual void  OnLoadError(const std::string & a_string)
        {  Form->OnLoadError(a_string);  }
    virtual void  OnLoadComplete()
        {  Form->OnLoadComplete();  }
    virtual void  OnRevision(const std::string & msg)
        {  Form->Revision(msg);  }

private:
    // Data
    MainWindow *       Form;
};
#endif

//===========================================================================
//  CLASS MainWindow  -- Form methods
//===========================================================================

//---------------------------------------------------------------------------
//  MainWindow::MainWindow() --  Ctor
//---------------------------------------------------------------------------

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    UI = new UserInterface(this);
    Io = new ApplicationIo(UI);

    int Count = Io->BoardCount();
    if (!Count)
        {
        Log("No baseboards enumerated!");
        }
    else
        {
        for (int i = 0; i < Count; ++i)
            AddToControl(ui->TargetComboBox, IntToString(i));

        ControlEnabled(ui->TargetComboBox, true);
        ToControl(ui->TargetComboBox, 0);
        }

    stringstream msg;
    msg << Io->BoardName() << " Logic Loader";
    ToCaption(this, msg.str());

    SetSettings();
}

//---------------------------------------------------------------------------
//  MainWindow::~MainWindow() --  Dtor
//---------------------------------------------------------------------------

MainWindow::~MainWindow()
{
    GetSettings();

    delete Io;
    Io = 0;

    delete UI;
    UI = 0;

    delete ui;
}


void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

//---------------------------------------------------------------------------
//  MainWindow::on_OpenButton_clicked() --
//---------------------------------------------------------------------------

void MainWindow::on_OpenButton_clicked()
{
    Io->Open();
    UpdateControls();
}

//---------------------------------------------------------------------------
//  MainWindow::on_CloseButton_clicked() --
//---------------------------------------------------------------------------

void MainWindow::on_CloseButton_clicked()
{
    Io->Close();
    UpdateControls();
}


void MainWindow::on_LogicFileButton_clicked()
{
    SelectFileDialog(ui->ImageFileEdit, "Image Files *.BIT *.EXO");
}

//---------------------------------------------------------------------------
//  MainWindow::on_LoadButton_clicked() --
//------------------------------------------------------------------------------

void MainWindow::on_LoadButton_clicked()
{
    Log("-----------------------------------------------");
    Log("Parsing logic file");

    ToControl(ui->ProgressBar, 0);

    std::string Image;
    FromControl(ui->ImageFileEdit, Image);
    Io->ConfigureFpga(Image);
}


//---------------------------------------------------------------------------
//  MainWindow::OnProgress() --
//------------------------------------------------------------------------------

void MainWindow::OnProgress(int percent)
{
    ToControl(ui->ProgressBar, percent);
}

//------------------------------------------------------------------------------
//  MainWindow::OnParseComplete() --
//------------------------------------------------------------------------------

void MainWindow::OnParseComplete()
{
    ToControl(ui->ProgressBar, 100);
    Log("Parse completed ok");
}

//------------------------------------------------------------------------------
//  MainWindow::OnLoadComplete() --
//------------------------------------------------------------------------------

void MainWindow::OnLoadComplete()
{
    ToControl(ui->ProgressBar, 100);
    Log("Load completed ok");

    Io->Sleep((unsigned int)1000);

    if (Io->Settings.Auto)
        Terminate();
}


//------------------------------------------------------------------------------
//  MainWindow::OnLoadError() --
//------------------------------------------------------------------------------

void MainWindow::OnLoadError(const std::string & a_string)
{
    Log(a_string.c_str());
    Io->Settings.Auto = false;
}

//===========================================================================
//  Support Functions
//===========================================================================

//---------------------------------------------------------------------------
//  MainWindow::OnProgress() --
//------------------------------------------------------------------------------

void MainWindow::UpdateControls()
{
    ControlEnabled(ui->OpenButton, !Io->IsOpen());
    ControlEnabled(ui->CloseButton, Io->IsOpen());
    ControlEnabled(ui->TargetComboBox, !Io->IsOpen());
    ControlEnabled(ui->ImageFileEdit, Io->IsOpen());
    ControlEnabled(ui->LoadButton, Io->IsOpen());
}

//---------------------------------------------------------------------------
//  MainWindow::Log() --
//---------------------------------------------------------------------------

void  MainWindow::Log(const std::string & a_string)
{
    AddToControl(ui->LogicLoadMemo, a_string);
}

//---------------------------------------------------------------------------
//  MainWindow::Revision() --
//---------------------------------------------------------------------------

void  MainWindow::Revision(const std::string & a_string)
{
    ToControl(ui->StatusBar, 0, a_string);
}

//---------------------------------------------------------------------------
//  MainWindow::GetSettings() --  Form controls to options object
//---------------------------------------------------------------------------

void MainWindow::GetSettings()
{
    FromControl(ui->TargetComboBox, Io->Settings.Target);
    FromControl(ui->ImageFileEdit, Io->Settings.ImageFile);
}

//---------------------------------------------------------------------------
//  MainWindow::SetSettings() --  Settings object to form controls
//---------------------------------------------------------------------------

void MainWindow::SetSettings()
{
    ToControl(ui->TargetComboBox, Io->Settings.Target);
    ToControl(ui->ImageFileEdit, Io->Settings.ImageFile);
}
