#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <ApplicationIo.h>
#include <Framework/Qt/GuiXfr_Qt.h>
#include <Framework/GuiLoader_Mb.h>
#include <sstream>

namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow {
    Q_OBJECT
public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

protected:
    void changeEvent(QEvent *e);

private:
    Ui::MainWindow *ui;

    friend class UserInterface;

private slots:
    void on_LogicLoadConfigBtn_clicked();
    void on_LogicLoadBrowseBtn_clicked();
    void on_ZbtTestButton_clicked();
    void on_DefaultRomButton_clicked();
    void on_ClockSourceComboBox_currentIndexChanged(int index);
    void on_FiclCommandEdit_returnPressed();
    void on_HelpButton_clicked();
    void on_DeviceWriteButton_clicked();
    void on_DeviceReadButton_clicked();
    void on_PortWriteButton_clicked();
    void on_PortReadButton_clicked();
    void on_LogicWriteButton_clicked();
    void on_LogicReadButton_clicked();
    void on_SoftwareAlertButton_clicked();
    void on_DebugScriptExecuteButton_clicked();
    void on_DebugScriptBrowseButton_clicked();
    void on_StoreToRomButton_clicked();
    void on_LoadFromRomButton_clicked();
    void on_StreamStopBtn_clicked();
    void on_StreamStartBtn_clicked();
    void on_CloseBtn_clicked();
    void on_OpenBtn_clicked();

    void on_CalculatExtClockRateBtn_clicked();

private: // User declarations
    //
    // Application Logic/Hardware I/O Logic
    IUserInterface *    Glob;
    ApplicationIo *     Io;
    Innovative::GuiLoader GL;
    std::stringstream   FiclText;
    QStatusBar *        PerformanceStatusBar;
    QStatusBar *        LogicStatusBar;

    vector<string>      BoardNames;

    //  Ficl Interface
    void         FiclInit();
    std::string  FiclReadCommandString();
    void         FiclCommandComplete();

    void  Log(const std::string & a_string);
    void  LogicStatus(const std::string & a_string);
    void  Status(const std::string & a_string);
    void  Warning(const std::string & a_string);
    void  PeriodicStatus();
    void  ExternalClockFrequency(float value);

    void  UpdateLogicLoadProgress(int percent);
    void  AfterStreamAutoStop();
    void  InstallSettings();
    void  SetSettings();
    void  GetSettings();
    void  DisplayHelp()
            {  /*Forms::Application->HelpContext(10);*/  }

    void  AdjustEnables();

};

//===========================================================================
//  CLASS UserInterface  -- Forward UI callbacks to Main App Form
//===========================================================================

class UserInterface : public IUserInterface
{
public:
        UserInterface(MainWindow * form)
                :  Form(form)
                {}

        void  UpdateLogicLoadProgress(int percent)
            {  Form->UpdateLogicLoadProgress(percent);  }
        void  Log(const std::string & a_string)
            {  Form->Log(a_string);  }
        void  Status(const std::string & a_string)
            {  Form->Status(a_string);  }
        void  LogicStatus(const std::string & a_string)
            {  Form->LogicStatus(a_string);  }
        void  GetSettings()
            {  Form->GetSettings();  }
        void  SetSettings()
            {  Form->SetSettings();  }
        void  AfterStreamAutoStop()
            {  Form->AfterStreamAutoStop();  }
        void  Warning(const std::string & a_string)
            {  Form->Warning(a_string);  }
        void  PeriodicStatus()
            {  Form->PeriodicStatus();  }
        void  DisplayHelp()
            {  /*Forms::Application->HelpContext(10);*/  }

        //  FICL User Interface Ftns
        void  FiclLog(const std::string & a_string)
                {  Log(a_string);  }
        std::string  FiclReadCommandString()
                {  return Form->FiclReadCommandString();  }
        void  FiclCommandComplete()
                {  Form->FiclCommandComplete();  }

private:
        MainWindow *  Form;
};


#endif // MAINWINDOW_H
